#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Ansible module to manage CheckPoint Firewall (c) 2019
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {
    "metadata_version": "1.1",
    "status": ["preview"],
    "supported_by": "community",
}

DOCUMENTATION = """
---
module: cp_mgmt_nat_rule_facts
short_description: Get nat-rule objects facts on Checkpoint over Web Services API
description:
  - Get nat-rule objects facts on Checkpoint devices.
  - All operations are performed over Web Services API.
  - This module handles both operations, get a specific object and get several objects,
    For getting a specific object use the parameter 'name'.
  - Available from R80 management version.
version_added: "2.0.0"
author: "Or Soffer (@chkp-orso)"
options:
  rule_number:
    description:
      - Rule number.
    type: str
  name:
    description:
      - Rule name.
        This parameter is relevant only for getting a specific object. Minimum version required is 1.7.1.
      - Available from R81 management version.
    type: str
  package:
    description:
      - Name of the package.
    type: str
  details_level:
    description:
      - The level of detail for some of the fields in the response can vary from showing only the UID value of the object to a fully detailed
        representation of the object.
    type: str
    choices: ['uid', 'standard', 'full']
  filter:
    description:
      - Search expression to filter the rulebase. The provided text should be exactly the same as it would be given in Smart Console. The logical
        operators in the expression ('AND', 'OR') should be provided in capital letters. If an operator is not used, the default OR operator applies.
      - Available from R80.10 management version.
    type: str
  filter_settings:
    description:
      - Sets filter preferences.
    type: dict
    suboptions:
      search_mode:
        description:
          - When set to 'general', both the Full Text Search and Packet Search are enabled. In this mode, Packet Search will not match on 'Any'
            object, a negated cell or a group-with-exclusion. When the search-mode is set to 'packet', by default, the match on 'Any' object, a negated cell
            or a group-with-exclusion are enabled. packet-search-settings may be provided to change the default behavior.
        type: str
        choices: ['general', 'packet']
      packet_search_settings:
        description:
          - When 'search-mode' is set to 'packet', this object allows to set the packet search preferences.
        type: dict
        suboptions:
          expand_group_members:
            description:
              - When true, if the search expression contains a UID or a name of a group object, results will include rules that match on at
                least one member of the group.
              - Available from R80.20 management version.
            type: bool
          expand_group_with_exclusion_members:
            description:
              - When true, if the search expression contains a UID or a name of a group-with-exclusion object, results will include rules that
                match at least one member of the "include" part and is not a member of the "except" part.
              - Available from R80.20 management version.
            type: bool
          match_on_any:
            description:
              - Whether to match on 'Any' object.
            type: bool
          match_on_group_with_exclusion:
            description:
              - Whether to match on a group-with-exclusion.
            type: bool
          match_on_negate:
            description:
              - Whether to match on a negated cell.
            type: bool
  limit:
    description:
      - The maximal number of returned results.
        This parameter is relevant only for getting few objects.
    type: int
  offset:
    description:
      - Number of the results to initially skip.
        This parameter is relevant only for getting few objects.
    type: int
  order:
    description:
      - Sorts the results by search criteria. Automatically sorts the results by Name, in the ascending order.
        This parameter is relevant only for getting few objects.
    type: list
    elements: dict
    suboptions:
      ASC:
        description:
          - Sorts results by the given field in ascending order.
        type: str
        choices: ['name']
      DESC:
        description:
          - Sorts results by the given field in descending order.
        type: str
        choices: ['name']
  use_object_dictionary:
    description:
      - N/A
    type: bool
  dereference_group_members:
    description:
      - Indicates whether to dereference "members" field by details level for every object in reply.
      - Available from R80.10 management version.
    type: bool
  show_membership:
    description:
      - Indicates whether to calculate and show "groups" field for every object in reply.
      - Available from R80.10 management version.
    type: bool
extends_documentation_fragment: check_point.mgmt.checkpoint_facts
"""

EXAMPLES = """
- name: show-nat-rule
  cp_mgmt_nat_rule_facts:
    package: standard
    name: nat_rule1

- name: show-nat-rulebase
  cp_mgmt_nat_rule_facts:
    details_level: standard
    limit: 2
    offset: 1
    package: standard
    use_object_dictionary: true
"""

RETURN = """
ansible_facts:
  description: The checkpoint object facts.
  returned: always.
  type: dict
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.check_point.mgmt.plugins.module_utils.checkpoint import (
    checkpoint_argument_spec_for_facts,
    api_call_facts_for_rule,
)


def main():
    argument_spec = dict(
        rule_number=dict(type="str"),
        name=dict(type='str'),
        package=dict(type="str"),
        details_level=dict(type="str", choices=["uid", "standard", "full"]),
        filter=dict(type="str"),
        filter_settings=dict(
            type="dict",
            options=dict(
                search_mode=dict(type="str", choices=["general", "packet"]),
                packet_search_settings=dict(
                    type="dict",
                    options=dict(
                        expand_group_members=dict(type="bool"),
                        expand_group_with_exclusion_members=dict(type="bool"),
                        match_on_any=dict(type="bool"),
                        match_on_group_with_exclusion=dict(type="bool"),
                        match_on_negate=dict(type="bool"),
                    ),
                ),
            ),
        ),
        limit=dict(type="int"),
        offset=dict(type="int"),
        order=dict(
            type="list",
            elements="dict",
            options=dict(
                ASC=dict(type="str", choices=["name"]),
                DESC=dict(type="str", choices=["name"]),
            ),
        ),
        use_object_dictionary=dict(type="bool"),
        dereference_group_members=dict(type="bool"),
        show_membership=dict(type="bool"),
    )
    argument_spec.update(checkpoint_argument_spec_for_facts)

    module = AnsibleModule(
        argument_spec=argument_spec, supports_check_mode=True
    )

    api_call_object = "nat-rule"
    api_call_object_plural_version = "nat-rulebase"

    result = api_call_facts_for_rule(
        module, api_call_object, api_call_object_plural_version
    )
    module.exit_json(ansible_facts=result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: organizations_devices_availabilities_change_history_info
short_description: Information module for organizations _devices _availabilities _change
  _history
description:
  - Get all organizations _devices _availabilities _change _history.
  - List the availability history information for devices in an organization.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module_info
  - cisco.meraki.module_info_pagination
author: Francisco Munoz (@fmunoz)
options:
  headers:
    description: Additional headers.
    type: dict
  organizationId:
    description:
      - OrganizationId path parameter. Organization ID.
    type: str
  perPage:
    description:
      - PerPage query parameter. The number of entries per page returned. Acceptable
        range is 3 - 1000. Default is 1000.
    type: int
  startingAfter:
    description:
      - >
        StartingAfter query parameter. A token used by the server to indicate the
        start of the page. Often this is a timestamp or an ID but it is not limited
        to those. This parameter should not be defined by client applications. The
        link for the first, last, prev, or next page in the HTTP Link header should
        define it.
    type: str
  endingBefore:
    description:
      - >
        EndingBefore query parameter. A token used by the server to indicate the end
        of the page. Often this is a timestamp or an ID but it is not limited to those.
        This parameter should not be defined by client applications. The link for
        the first, last, prev, or next page in the HTTP Link header should define
        it.
    type: str
  t0:
    description:
      - T0 query parameter. The beginning of the timespan for the data. The maximum
        lookback period is 31 days from today.
    type: str
  t1:
    description:
      - T1 query parameter. The end of the timespan for the data. T1 can be a maximum
        of 31 days after t0.
    type: str
  timespan:
    description:
      - >
        Timespan query parameter. The timespan for which the information will be fetched.
        If specifying timespan, do not specify parameters t0 and t1. The value must
        be in seconds and be less than or equal to 31 days. The default is 1 day.
    type: float
  serials:
    description:
      - Serials query parameter. Optional parameter to filter device availabilities
        history by device serial numbers.
    elements: str
    type: list
  productTypes:
    description:
      - ProductTypes query parameter. Optional parameter to filter device availabilities
        history by device product types.
    elements: str
    type: list
  networkIds:
    description:
      - NetworkIds query parameter. Optional parameter to filter device availabilities
        history by network IDs.
    elements: str
    type: list
  statuses:
    description:
      - Statuses query parameter. Optional parameter to filter device availabilities
        history by device statuses.
    elements: str
    type: list
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for organizations getOrganizationDevicesAvailabilitiesChangeHistory
    description: Complete reference of the getOrganizationDevicesAvailabilitiesChangeHistory
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!get-organization-devices-availabilities-change-history
notes:
  - SDK Method used are
    organizations.Organizations.get_organization_devices_availabilities_change_history,
  - Paths used are
    get /organizations/{organizationId}/devices/availabilities/changeHistory,
"""

EXAMPLES = r"""
- name: Get all organizations _devices _availabilities _change _history
  cisco.meraki.organizations_devices_availabilities_change_history_info:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    perPage: 0
    startingAfter: string
    endingBefore: string
    t0: string
    t1: string
    timespan: 0
    serials: []
    productTypes: []
    networkIds: []
    statuses: []
    organizationId: string
    total_pages: -1
    direction: next
  register: result
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: list
  elements: dict
  sample: >
    [
      {
        "ts": "string",
        "device": {
          "serial": "string",
          "name": "string",
          "productType": "string",
          "model": "string"
        },
        "details": {
          "old": [
            {
              "name": "string",
              "value": "string"
            }
          ],
          "new": [
            {
              "name": "string",
              "value": "string"
            }
          ]
        },
        "network": {
          "id": "string",
          "name": "string",
          "url": "string",
          "tags": [
            "string"
          ]
        }
      }
    ]
"""



#!/usr/bin/python

# Copyright (c) 2013, Daniel Jaouen <dcj24@cornell.edu>
# Copyright (c) 2016, Indrajit Raychaudhuri <irc+code@indrajit.com>
#
# Based on homebrew (Andrew Dunham <andrew@du.nham.ca>)
#
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: homebrew_tap
author:
  - "Indrajit Raychaudhuri (@indrajitr)"
  - "Daniel Jaouen (@danieljaouen)"
short_description: Tap a Homebrew repository
description:
  - Tap external Homebrew repositories.
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  name:
    description:
      - The GitHub user/organization repository to tap.
    required: true
    aliases: ['tap']
    type: list
    elements: str
  url:
    description:
      - The optional git URL of the repository to tap. The URL is not assumed to be on GitHub, and the protocol does not have
        to be HTTP. Any location and protocol that git can handle is fine.
      - O(name) option may not be a list of multiple taps (but a single tap instead) when this option is provided.
    type: str
  state:
    description:
      - State of the repository.
    choices: ['present', 'absent']
    default: 'present'
    type: str
  path:
    description:
      - A V(:) separated list of paths to search for C(brew) executable.
    default: '/usr/local/bin:/opt/homebrew/bin:/home/linuxbrew/.linuxbrew/bin'
    type: path
    version_added: '2.1.0'
requirements: [homebrew]
"""

EXAMPLES = r"""
- name: Tap a Homebrew repository, state present
  community.general.homebrew_tap:
    name: homebrew/dupes

- name: Tap a Homebrew repository, state absent
  community.general.homebrew_tap:
    name: homebrew/dupes
    state: absent

- name: Tap a Homebrew repository, state present
  community.general.homebrew_tap:
    name: homebrew/dupes,homebrew/science
    state: present

- name: Tap a Homebrew repository using url, state present
  community.general.homebrew_tap:
    name: telemachus/brew
    url: 'https://bitbucket.org/telemachus/brew'
"""

import re

from ansible.module_utils.basic import AnsibleModule


def a_valid_tap(tap):
    """Returns True if the tap is valid."""
    regex = re.compile(r"^([\w-]+)/(homebrew-)?([\w-]+)$")
    return regex.match(tap)


def already_tapped(module, brew_path, tap, taps=None):
    """Returns True if already tapped."""
    if taps is None:
        rc, out, err = module.run_command([brew_path, "tap"])
        taps = [tap_.strip().lower() for tap_ in out.split("\n") if tap_]
    tap_name = re.sub("homebrew-", "", tap.lower())
    return tap_name in taps


def add_tap(module, brew_path, tap, url=None, taps=None):
    """Adds a single tap."""
    failed, changed, msg = False, False, ""

    if not a_valid_tap(tap):
        failed = True
        msg = f"not a valid tap: {tap}"

    elif not already_tapped(module, brew_path, tap, taps):
        if module.check_mode:
            module.exit_json(changed=True)

        cmd = [brew_path, "tap", tap]
        if url:
            cmd.append(url)
        rc, out, err = module.run_command(cmd)
        if rc == 0:
            changed = True
            msg = f"successfully tapped: {tap}"
        else:
            failed = True
            msg = f"failed to tap: {tap} due to {err}"

    else:
        msg = f"already tapped: {tap}"

    return (failed, changed, msg)


def add_taps(module, brew_path, taps):
    """Adds one or more taps."""
    failed, changed, unchanged, added, msg = False, False, 0, 0, ""

    rc, out, err = module.run_command([brew_path, "tap"])
    tapped = [t.strip().lower() for t in out.split("\n") if t]

    for tap in taps:
        (failed, changed, msg) = add_tap(module, brew_path, tap, taps=tapped)
        if failed:
            break
        if changed:
            added += 1
        else:
            unchanged += 1

    if failed:
        msg = f"added: {added}, unchanged: {unchanged}, error: {msg}"
    elif added:
        changed = True
        msg = f"added: {added}, unchanged: {unchanged}"
    else:
        msg = f"added: {added}, unchanged: {unchanged}"

    return (failed, changed, msg)


def remove_tap(module, brew_path, tap, taps=None):
    """Removes a single tap."""
    failed, changed, msg = False, False, ""

    if not a_valid_tap(tap):
        failed = True
        msg = f"not a valid tap: {tap}"

    elif already_tapped(module, brew_path, tap, taps):
        if module.check_mode:
            module.exit_json(changed=True)

        rc, out, err = module.run_command([brew_path, "untap", tap])
        if not already_tapped(module, brew_path, tap):
            changed = True
            msg = f"successfully untapped: {tap}"
        else:
            failed = True
            msg = f"failed to untap: {tap} due to {err}"

    else:
        msg = f"already untapped: {tap}"

    return (failed, changed, msg)


def remove_taps(module, brew_path, taps):
    """Removes one or more taps."""
    failed, changed, unchanged, removed, msg = False, False, 0, 0, ""

    rc, out, err = module.run_command([brew_path, "tap"])
    tapped = [t.strip().lower() for t in out.split("\n") if t]

    for tap in taps:
        (failed, changed, msg) = remove_tap(module, brew_path, tap, taps=tapped)
        if failed:
            break
        if changed:
            removed += 1
        else:
            unchanged += 1

    if failed:
        msg = f"removed: {removed}, unchanged: {unchanged}, error: {msg}"
    elif removed:
        changed = True
        msg = f"removed: {removed}, unchanged: {unchanged}"
    else:
        msg = f"removed: {removed}, unchanged: {unchanged}"

    return (failed, changed, msg)


def main():
    module = AnsibleModule(
        argument_spec=dict(
            name=dict(aliases=["tap"], type="list", required=True, elements="str"),
            url=dict(),
            state=dict(default="present", choices=["present", "absent"]),
            path=dict(
                default="/usr/local/bin:/opt/homebrew/bin:/home/linuxbrew/.linuxbrew/bin",
                type="path",
            ),
        ),
        supports_check_mode=True,
    )

    path = module.params["path"]
    if path:
        path = path.split(":")

    brew_path = module.get_bin_path(
        "brew",
        required=True,
        opt_dirs=path,
    )

    taps = module.params["name"]
    url = module.params["url"]

    if module.params["state"] == "present":
        if url is None:
            # No tap URL provided explicitly, continue with bulk addition
            # of all the taps.
            failed, changed, msg = add_taps(module, brew_path, taps)
        else:
            # When an tap URL is provided explicitly, we allow adding
            # *single* tap only. Validate and proceed to add single tap.
            if len(taps) > 1:
                msg = "List of multiple taps may not be provided with 'url' option."
                module.fail_json(msg=msg)
            else:
                failed, changed, msg = add_tap(module, brew_path, taps[0], url)

        if failed:
            module.fail_json(msg=msg)
        else:
            module.exit_json(changed=changed, msg=msg)

    elif module.params["state"] == "absent":
        failed, changed, msg = remove_taps(module, brew_path, taps)

        if failed:
            module.fail_json(msg=msg)
        else:
            module.exit_json(changed=changed, msg=msg)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# Copyright (c) 2016-2017, Yanis Guenane <yanis+ansible@guenane.org>
# Copyright (c) 2017, Markus Teufelberger <mteufelberger+ansible@mgit.at>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: x509_certificate
short_description: Generate and/or check OpenSSL certificates
description:
  - It implements a notion of provider (one of V(selfsigned), V(ownca), and V(acme)) for your certificate.
  - Please note that the module regenerates existing certificate if it does not match the module's options, or if it seems
    to be corrupt. If you are concerned that this could overwrite your existing certificate, consider using the O(backup)
    option.
  - Note that this module was called C(openssl_certificate) when included directly in Ansible up to version 2.9. When moved
    to the collection C(community.crypto), it was renamed to M(community.crypto.x509_certificate). From Ansible 2.10 on, it
    can still be used by the old short name (or by C(ansible.builtin.openssl_certificate)), which redirects to M(community.crypto.x509_certificate).
    When using FQCNs or when using the L(collections,https://docs.ansible.com/ansible/latest/user_guide/collections_using.html#using-collections-in-a-playbook)
    keyword, the new name M(community.crypto.x509_certificate) should be used to avoid a deprecation warning.
author:
  - Yanis Guenane (@Spredzy)
  - Markus Teufelberger (@MarkusTeufelberger)
extends_documentation_fragment:
  - ansible.builtin.files
  - community.crypto._attributes
  - community.crypto._attributes.files
  - community.crypto._module_certificate
  - community.crypto._module_certificate.backend_acme_documentation
  - community.crypto._module_certificate.backend_ownca_documentation
  - community.crypto._module_certificate.backend_selfsigned_documentation
attributes:
  check_mode:
    support: full
  safe_file_operations:
    support: full
options:
  state:
    description:
      - Whether the certificate should exist or not, taking action if the state is different from what is stated.
    type: str
    default: present
    choices: [absent, present]

  path:
    description:
      - Remote absolute path where the generated certificate file should be created or is already located.
    type: path
    required: true

  provider:
    description:
      - Name of the provider to use to generate/retrieve the OpenSSL certificate. Please see the examples on how to emulate
        it with M(community.crypto.x509_certificate_info), M(community.crypto.openssl_csr_info), M(community.crypto.openssl_privatekey_info)
        and M(ansible.builtin.assert).
      - Required if O(state) is V(present).
      - The V(entrust) provider has been removed from community.crypto 3.0.0 due to sunsetting of the ECS API.
    type: str
    choices: [acme, ownca, selfsigned]

  return_content:
    description:
      - If set to V(true), will return the (current or generated) certificate's content as RV(certificate).
    type: bool
    default: false
    version_added: '1.0.0'

  backup:
    description:
      - Create a backup file including a timestamp so you can get the original certificate back if you overwrote it with a
        new one by accident.
    type: bool
    default: false

  csr_content:
    version_added: '1.0.0'
  privatekey_content:
    version_added: '1.0.0'
  acme_directory:
    version_added: '1.0.0'
  ownca_content:
    version_added: '1.0.0'
  ownca_privatekey_content:
    version_added: '1.0.0'

seealso:
  - module: community.crypto.x509_certificate_pipe
"""

EXAMPLES = r"""
---
- name: Generate a Self Signed OpenSSL certificate
  community.crypto.x509_certificate:
    path: /etc/ssl/crt/ansible.com.crt
    privatekey_path: /etc/ssl/private/ansible.com.pem
    csr_path: /etc/ssl/csr/ansible.com.csr
    provider: selfsigned

- name: Generate an OpenSSL certificate signed with your own CA certificate
  community.crypto.x509_certificate:
    path: /etc/ssl/crt/ansible.com.crt
    csr_path: /etc/ssl/csr/ansible.com.csr
    ownca_path: /etc/ssl/crt/ansible_CA.crt
    ownca_privatekey_path: /etc/ssl/private/ansible_CA.pem
    provider: ownca

- name: Generate a Let's Encrypt Certificate
  community.crypto.x509_certificate:
    path: /etc/ssl/crt/ansible.com.crt
    csr_path: /etc/ssl/csr/ansible.com.csr
    provider: acme
    acme_accountkey_path: /etc/ssl/private/ansible.com.pem
    acme_challenge_path: /etc/ssl/challenges/ansible.com/

- name: Force (re-)generate a new Let's Encrypt Certificate
  community.crypto.x509_certificate:
    path: /etc/ssl/crt/ansible.com.crt
    csr_path: /etc/ssl/csr/ansible.com.csr
    provider: acme
    acme_accountkey_path: /etc/ssl/private/ansible.com.pem
    acme_challenge_path: /etc/ssl/challenges/ansible.com/
    force: true

# The following example shows how to emulate the behavior of the removed
# "assertonly" provider with the x509_certificate_info, openssl_csr_info,
# openssl_privatekey_info and assert modules:

- name: Get certificate information
  community.crypto.x509_certificate_info:
    path: /etc/ssl/crt/ansible.com.crt
    # for valid_at, invalid_at and valid_in
    valid_at:
      one_day_ten_hours: "+1d10h"
      fixed_timestamp: 20200331202428Z
      ten_seconds: "+10"
  register: result

- name: Get CSR information
  community.crypto.openssl_csr_info:
    # Verifies that the CSR signature is valid; module will fail if not
    path: /etc/ssl/csr/ansible.com.csr
  register: result_csr

- name: Get private key information
  community.crypto.openssl_privatekey_info:
    path: /etc/ssl/csr/ansible.com.key
  register: result_privatekey

- name: Check conditions on certificate, CSR, and private key
  ansible.builtin.assert:
    that:
      # When private key was specified for assertonly, this was checked:
      - result.public_key == result_privatekey.public_key
      # When CSR was specified for assertonly, this was checked:
      - result.public_key == result_csr.public_key
      - result.subject_ordered == result_csr.subject_ordered
      - result.extensions_by_oid == result_csr.extensions_by_oid
      # signature_algorithms check
      - "result.signature_algorithm == 'sha256WithRSAEncryption' or result.signature_algorithm == 'sha512WithRSAEncryption'"
      # subject and subject_strict
      - "result.subject.commonName == 'ansible.com'"
      - "result.subject | length == 1" # the number must be the number of entries you check for
      # issuer and issuer_strict
      - "result.issuer.commonName == 'ansible.com'"
      - "result.issuer | length == 1" # the number must be the number of entries you check for
      # has_expired
      - not result.expired
      # version
      - result.version == 3
      # key_usage and key_usage_strict
      - "'Data Encipherment' in result.key_usage"
      - "result.key_usage | length == 1" # the number must be the number of entries you check for
      # extended_key_usage and extended_key_usage_strict
      - "'DVCS' in result.extended_key_usage"
      - "result.extended_key_usage | length == 1" # the number must be the number of entries you check for
      # subject_alt_name and subject_alt_name_strict
      - "'dns:ansible.com' in result.subject_alt_name"
      - "result.subject_alt_name | length == 1" # the number must be the number of entries you check for
      # not_before and not_after
      - "result.not_before == '20190331202428Z'"
      - "result.not_after == '20190413202428Z'"
      # valid_at, invalid_at and valid_in
      - "result.valid_at.one_day_ten_hours" # for valid_at
      - "not result.valid_at.fixed_timestamp" # for invalid_at
      - "result.valid_at.ten_seconds" # for valid_in
"""

RETURN = r"""
filename:
  description: Path to the generated certificate.
  returned: changed or success
  type: str
  sample: /etc/ssl/crt/www.ansible.com.crt
backup_file:
  description: Name of backup file created.
  returned: changed and if O(backup) is V(true)
  type: str
  sample: /path/to/www.ansible.com.crt.2019-03-09@11:22~
certificate:
  description: The (current or generated) certificate's content.
  returned: if O(state) is V(present) and O(return_content) is V(true)
  type: str
  version_added: '1.0.0'
"""


import os
import typing as t

from ansible_collections.community.crypto.plugins.module_utils._crypto.basic import (
    OpenSSLObjectError,
)
from ansible_collections.community.crypto.plugins.module_utils._crypto.module_backends.certificate import (
    get_certificate_argument_spec,
    select_backend,
)
from ansible_collections.community.crypto.plugins.module_utils._crypto.module_backends.certificate_acme import (
    AcmeCertificateProvider,
    add_acme_provider_to_argument_spec,
)
from ansible_collections.community.crypto.plugins.module_utils._crypto.module_backends.certificate_ownca import (
    OwnCACertificateProvider,
    add_ownca_provider_to_argument_spec,
)
from ansible_collections.community.crypto.plugins.module_utils._crypto.module_backends.certificate_selfsigned import (
    SelfSignedCertificateProvider,
    add_selfsigned_provider_to_argument_spec,
)
from ansible_collections.community.crypto.plugins.module_utils._crypto.support import (
    OpenSSLObject,
)
from ansible_collections.community.crypto.plugins.module_utils._io import (
    load_file_if_exists,
    write_file,
)

if t.TYPE_CHECKING:
    from ansible.module_utils.basic import AnsibleModule  # pragma: no cover

    from ansible_collections.community.crypto.plugins.module_utils._crypto.module_backends.certificate import (  # pragma: no cover
        CertificateBackend,
    )


class CertificateAbsent(OpenSSLObject):
    def __init__(self, module: AnsibleModule) -> None:
        super().__init__(
            path=module.params["path"],
            state=module.params["state"],
            force=module.params["force"],
            check_mode=module.check_mode,
        )
        self.module = module
        self.return_content: bool = module.params["return_content"]
        self.backup: bool = module.params["backup"]
        self.backup_file: str | None = None

    def generate(self, module: AnsibleModule) -> None:
        pass

    def remove(self, module: AnsibleModule) -> None:
        if self.backup:
            self.backup_file = module.backup_local(self.path)
        super().remove(module)

    def dump(self, check_mode: bool = False) -> dict[str, t.Any]:
        result = {
            "changed": self.changed,
            "filename": self.path,
            "privatekey": self.module.params["privatekey_path"],
            "csr": self.module.params["csr_path"],
        }
        if self.backup_file:
            result["backup_file"] = self.backup_file
        if self.return_content:
            result["certificate"] = None

        return result


class GenericCertificate(OpenSSLObject):
    """Retrieve a certificate using the given module backend."""

    def __init__(self, module: AnsibleModule, module_backend: CertificateBackend):
        super().__init__(
            path=module.params["path"],
            state=module.params["state"],
            force=module.params["force"],
            check_mode=module.check_mode,
        )
        self.module = module
        self.return_content = module.params["return_content"]
        self.backup = module.params["backup"]
        self.backup_file: str | None = None

        self.module_backend = module_backend
        self.module_backend.set_existing(
            certificate_bytes=load_file_if_exists(path=self.path, module=module)
        )

    def generate(self, module: AnsibleModule) -> None:
        if self.module_backend.needs_regeneration():
            if not self.check_mode:
                self.module_backend.generate_certificate()
                result = self.module_backend.get_certificate_data()
                if self.backup:
                    self.backup_file = module.backup_local(self.path)
                write_file(module=module, content=result)
            self.changed = True

        file_args = module.load_file_common_arguments(module.params)
        if module.check_file_absent_if_check_mode(file_args["path"]):
            self.changed = True
        else:
            self.changed = module.set_fs_attributes_if_different(
                file_args, self.changed
            )

    def check(self, module: AnsibleModule, *, perms_required: bool = True) -> bool:
        """Ensure the resource is in its desired state."""
        return (
            super().check(module=module, perms_required=perms_required)
            and not self.module_backend.needs_regeneration()
        )

    def dump(self, check_mode: bool = False) -> dict[str, t.Any]:
        result = self.module_backend.dump(include_certificate=self.return_content)
        result.update(
            {
                "changed": self.changed,
                "filename": self.path,
            }
        )
        if self.backup_file:
            result["backup_file"] = self.backup_file
        return result


def main() -> t.NoReturn:
    argument_spec = get_certificate_argument_spec()
    add_acme_provider_to_argument_spec(argument_spec)
    add_ownca_provider_to_argument_spec(argument_spec)
    add_selfsigned_provider_to_argument_spec(argument_spec)
    argument_spec.argument_spec.update(
        {
            "state": {
                "type": "str",
                "default": "present",
                "choices": ["present", "absent"],
            },
            "path": {"type": "path", "required": True},
            "backup": {"type": "bool", "default": False},
            "return_content": {"type": "bool", "default": False},
        }
    )
    argument_spec.required_if.append(("state", "present", ["provider"]))
    module = argument_spec.create_ansible_module(
        add_file_common_args=True,
        supports_check_mode=True,
    )

    try:
        certificate: GenericCertificate | CertificateAbsent
        if module.params["state"] == "absent":
            certificate = CertificateAbsent(module)

            if module.check_mode:
                result = certificate.dump(check_mode=True)
                result["changed"] = os.path.exists(module.params["path"])
                module.exit_json(**result)

            certificate.remove(module)

        else:
            base_dir = os.path.dirname(module.params["path"]) or "."
            if not os.path.isdir(base_dir):
                module.fail_json(
                    name=base_dir,
                    msg=f"The directory {base_dir} does not exist or the file is not a directory",
                )

            provider = module.params["provider"]
            provider_map: dict[
                str,
                type[AcmeCertificateProvider]
                | type[OwnCACertificateProvider]
                | type[SelfSignedCertificateProvider],
            ] = {
                "acme": AcmeCertificateProvider,
                "ownca": OwnCACertificateProvider,
                "selfsigned": SelfSignedCertificateProvider,
            }

            module_backend = select_backend(
                module=module, provider=provider_map[provider]()
            )
            certificate = GenericCertificate(module, module_backend)
            certificate.generate(module)

        result = certificate.dump()
        module.exit_json(**result)
    except OpenSSLObjectError as exc:
        module.fail_json(msg=str(exc))


if __name__ == "__main__":
    main()



#!/usr/bin/python
# Copyright (c) 2017, Yanis Guenane <yanis+ansible@guenane.org>
# Copyright (c) 2020, Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: openssl_csr_pipe
short_description: Generate OpenSSL Certificate Signing Request (CSR)
version_added: 1.3.0
description:
  - Please note that the module regenerates an existing CSR if it does not match the module's options, or if it seems to be
    corrupt.
author:
  - Yanis Guenane (@Spredzy)
  - Felix Fontein (@felixfontein)
extends_documentation_fragment:
  - community.crypto._attributes
  - community.crypto._module_csr
attributes:
  check_mode:
    support: full
    details:
      - Since community.crypto 3.0.0, the module ignores check mode and always behaves as if check mode is not active.
options:
  content:
    description:
      - The existing CSR.
    type: str
  privatekey_path:
    description:
      - The path to the private key to use when signing the certificate signing request.
      - Either O(privatekey_path) or O(privatekey_content) must be specified, but not both.
  privatekey_content:
    description:
      - The content of the private key to use when signing the certificate signing request.
      - Either O(privatekey_path) or O(privatekey_content) must be specified, but not both.
seealso:
  - module: community.crypto.openssl_csr
"""

EXAMPLES = r"""
---
- name: Generate an OpenSSL Certificate Signing Request
  community.crypto.openssl_csr_pipe:
    privatekey_path: /etc/ssl/private/ansible.com.pem
    common_name: www.ansible.com
  register: result
- name: Print CSR
  ansible.builtin.debug:
    var: result.csr

- name: Generate an OpenSSL Certificate Signing Request with an inline CSR
  community.crypto.openssl_csr:
    content: "{{ lookup('ansible.builtin.file', '/etc/ssl/csr/www.ansible.com.csr') }}"
    privatekey_content: "{{ private_key_content }}"
    common_name: www.ansible.com
  register: result
- name: Store CSR
  ansible.builtin.copy:
    dest: /etc/ssl/csr/www.ansible.com.csr
    content: "{{ result.csr }}"
  when: result is changed
"""

RETURN = r"""
privatekey:
  description:
    - Path to the TLS/SSL private key the CSR was generated for.
    - Will be V(none) if the private key has been provided in O(privatekey_content).
  returned: changed or success
  type: str
  sample: /etc/ssl/private/ansible.com.pem
subject:
  description: A list of the subject tuples attached to the CSR.
  returned: changed or success
  type: list
  elements: list
  sample: [['CN', 'www.ansible.com'], ['O', 'Ansible']]
subjectAltName:
  description: The alternative names this CSR is valid for.
  returned: changed or success
  type: list
  elements: str
  sample: ['DNS:www.ansible.com', 'DNS:m.ansible.com']
keyUsage:
  description: Purpose for which the public key may be used.
  returned: changed or success
  type: list
  elements: str
  sample: ['digitalSignature', 'keyAgreement']
extendedKeyUsage:
  description: Additional restriction on the public key purposes.
  returned: changed or success
  type: list
  elements: str
  sample: ['clientAuth']
basicConstraints:
  description: Indicates if the certificate belongs to a CA.
  returned: changed or success
  type: list
  elements: str
  sample: ['CA:TRUE', 'pathLenConstraint:0']
ocsp_must_staple:
  description: Indicates whether the certificate has the OCSP Must Staple feature enabled.
  returned: changed or success
  type: bool
  sample: false
name_constraints_permitted:
  description: List of permitted subtrees to sign certificates for.
  returned: changed or success
  type: list
  elements: str
  sample: ['email:.somedomain.com']
name_constraints_excluded:
  description: List of excluded subtrees the CA cannot sign certificates for.
  returned: changed or success
  type: list
  elements: str
  sample: ['email:.com']
csr:
  description: The (current or generated) CSR's content.
  returned: changed or success
  type: str
"""

import typing as t

from ansible_collections.community.crypto.plugins.module_utils._crypto.basic import (
    OpenSSLObjectError,
)
from ansible_collections.community.crypto.plugins.module_utils._crypto.module_backends.csr import (
    get_csr_argument_spec,
    select_backend,
)

if t.TYPE_CHECKING:
    from ansible.module_utils.basic import AnsibleModule  # pragma: no cover

    from ansible_collections.community.crypto.plugins.module_utils._crypto.module_backends.csr import (  # pragma: no cover
        CertificateSigningRequestBackend,
    )


class CertificateSigningRequestModule:
    def __init__(
        self, module: AnsibleModule, module_backend: CertificateSigningRequestBackend
    ) -> None:
        self.check_mode = module.check_mode
        self.module = module
        self.module_backend = module_backend
        self.changed = False
        if module.params["content"] is not None:
            self.module_backend.set_existing(
                csr_bytes=module.params["content"].encode("utf-8")
            )

    def generate(self, module: AnsibleModule) -> None:
        """Generate the certificate signing request."""
        if self.module_backend.needs_regeneration():
            self.module_backend.generate_csr()
            self.changed = True

    def dump(self) -> dict[str, t.Any]:
        """Serialize the object into a dictionary."""
        result = self.module_backend.dump(include_csr=True)
        result.update(
            {
                "changed": self.changed,
            }
        )
        return result


def main() -> t.NoReturn:
    argument_spec = get_csr_argument_spec()
    argument_spec.argument_spec.update(
        {
            "content": {"type": "str"},
        }
    )
    module = argument_spec.create_ansible_module(
        supports_check_mode=True,
    )

    try:
        module_backend = select_backend(module)

        csr = CertificateSigningRequestModule(module, module_backend)
        csr.generate(module)
        result = csr.dump()
        module.exit_json(**result)
    except OpenSSLObjectError as exc:
        module.fail_json(msg=str(exc))


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2025, Akini Ross (@akinross) <akinross@cisco.com>

# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: ndo_l3out_floating_svi_interface
version_added: "2.12.0"
short_description: Manage L3Out Floating SVI Interfaces on Cisco Nexus Dashboard Orchestrator (NDO).
description:
- Manage L3Out Floating SVI Interfaces on Cisco Nexus Dashboard Orchestrator (NDO).
- This module is only supported on ND v3.1 (NDO v4.3) and later.
author:
- Akini Ross (@akinross)
options:
  template:
    description:
    - The name of the template.
    - The template must be an L3Out template.
    - This parameter or O(template_id) is required.
    type: str
    aliases: [ l3out_template ]
  template_id:
    description:
    - The ID of the L3Out template.
    - This parameter or O(template) is required.
    type: str
    aliases: [ l3out_template_id ]
  l3out:
    description:
    - The name of the L3Out.
    - This parameter or O(l3out_uuid) is required.
    type: str
    aliases: [ l3out_name ]
  l3out_uuid:
    description:
    - The UUID of the L3Out.
    - This parameter or O(l3out) is required.
    type: str
  node_id:
    description:
    - The ID of the node (border leaf switch) where to deploy the L3Out routing protocol and node-level protocol configurations.
    - When specified, the node configuration is created under the L3Out template when it does not exist.
    - When specified, the node configuration is updated under the L3Out template when it already exists.
    - When specified, the node configuration is deleted under the L3Out template when there are no interfaces referencing it.
    type: str
    aliases: [ node, border_leaf ]
  node_router_id:
    description:
    - The router ID of the node.
    - This parameter is required when the node configuration does not exist under the L3Out template.
    type: str
    aliases: [ router_id ]
  node_group_policy:
    description:
    - The name of the node group policy.
    type: str
  use_router_id_as_loopback:
    description:
    - Whether to use the router ID as the loopback address of the node.
    type: bool
  node_loopback_ip:
    description:
    - The loopback IP address of the node.
    type: str
    aliases: [ loopback_ip ]
  interface_group_policy:
    description:
    - The name of the interface group policy.
    type: str
  ipv4_address:
    description:
    - The IPv4 address of the interface.
    type: str
  ipv6_address:
    description:
    - The IPv6 address of the interface.
    type: str
  ipv6_link_local_address:
    description:
    - The IPv6 link-local address of the interface.
    type: str
  ipv6_dad:
    description:
    - Whether to enable IPv6 Duplicate Address Detection (DAD).
    - If this parameter is unspecified, NDO defaults to O(ipv6_dad=enabled).
    type: str
    choices: [ enabled, disabled ]
  mac:
    description:
    - The MAC address of the interface.
    type: str
  mtu:
    description:
    - The Maximum Transmission Unit (MTU) of the interface.
    - Use O(mtu=inherit) to inherit the value configured under the fabric L2 MTU settings.
    - The value must be 1, or in the range 576 - 9216 or O(mtu=inherit).
    type: str
  target_dscp:
    description:
    - The target Differentiated Services Code Point (DSCP) of the interface.
    - If this parameter is unspecified, NDO defaults to O(target_dscp=unspecified).
    type: str
    choices:
    - af11
    - af12
    - af13
    - af21
    - af22
    - af23
    - af31
    - af32
    - af33
    - af41
    - af42
    - af43
    - cs0
    - cs1
    - cs2
    - cs3
    - cs4
    - cs5
    - cs6
    - cs7
    - expedited_forwarding
    - unspecified
    - voice_admit
  encapsulation_scope:
    description:
    - The encapsulation scope of the interface.
    type: str
    aliases: [ encap_scope ]
    choices: [ local, vrf ]
  encapsulation_type:
    description:
    - The encapsulation type of the interface.
    type: str
    aliases: [ encap_type ]
    choices: [ vlan, vxlan ]
  encapsulation_value:
    description:
    - The encapsulation value of the interface.
    - The option O(encapsulation_type=vlan), specifies VLAN ID which must be in the range 1 - 4094.
    - The option O(encapsulation_type=vxlan), specifies VXLAN Network Identifier (VNI) which must be in the range 5000 - 16777215.
    type: int
    aliases: [ encap, encapsulation, encapsulation_id ]
  encapsulation_mode:
    description:
    - The encapsulation mode of the interface.
    type: str
    aliases: [ encap_mode ]
    choices: [ access, trunk, trunk_native ]
  auto_state:
    description:
    - Whether to enable auto state on the interface.
    - Controls whether the SVI comes up or down based on the state of its associated Layer 2 ports.
    type: str
    choices: [ enabled, disabled ]
  state:
    description:
    - Determines the desired state of the resource.
    - Use C(absent) to remove the resource.
    - Use C(query) to list the resource.
    - Use C(present) to create or update the resource.
    type: str
    choices: [ absent, query, present ]
    default: query
notes:
- The O(template) or O(template_id) must exist before using this module in your playbook.
  The M(cisco.mso.ndo_template) module can be used for this.
- The O(l3out) or O(l3out_uuid) must exist before using this module in your playbook.
  The M(cisco.mso.ndo_l3out_template) module can be used for this.
- The O(node_group_policy) must exist before using this module in your playbook.
  The M(cisco.mso.ndo_l3out_node_group_policy) module can be used for this.
- The O(interface_group_policy) must exist before using this module in your playbook.
  The M(cisco.mso.ndo_l3out_interface_group_policy) module can be used for this.
seealso:
- module: cisco.mso.ndo_template
- module: cisco.mso.ndo_l3out_template
- module: cisco.mso.ndo_l3out_node_group_policy
- module: cisco.mso.ndo_l3out_interface_group_policy
extends_documentation_fragment: cisco.mso.modules
"""

EXAMPLES = r"""
- name: Create a L3Out Floating SVI Interface with L3Out Node configuration
  cisco.mso.ndo_l3out_floating_svi_interface:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: l3out_template
    l3out: l3out_name
    node_id: 101
    node_router_id: 1.1.1.1
    ipv4_address: 10.0.0.2/24
    ipv6_address: 2::101/16
    ipv6_link_local_address: FE80::10
    ipv6_dad: disabled
    encapsulation_type: vlan
    encapsulation_value: 1200
    mac: 00:22:BD:F8:19:FE
    mtu: 1000
    interface_group_policy: interface_group_policy_1
    node_group_policy: node_group_policy_1
    target_dscp: af11
    encapsulation_scope: vrf
    encapsulation_mode: trunk
    auto_state: enabled
    state: present

- name: Update a L3Out Floating SVI Interface and L3Out Node configuration
  cisco.mso.ndo_l3out_floating_svi_interface:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template_id: "{{ l3out_template.current.templateId }}"
    l3out_uuid: "{{ l3out.current.uuid }}"
    node_id: 101
    node_router_id: 1.1.1.1
    encapsulation_type: vlan
    encapsulation_value: 1200
    ipv4_address: 10.0.1.1/24
    ipv6_address: 1::102/16
    ipv6_dad: disabled
    ipv6_link_local_address: FE80::12
    mac: 00:22:BD:F8:19:CC
    mtu: 1800
    interface_group_policy: interface_group_policy_2
    node_group_policy: node_group_policy_2
    target_dscp: af41
    encapsulation_scope: local
    encapsulation_mode: access
    auto_state: enabled
    state: present

- name: Query an existing L3Out Floating SVI Interface with L3Out Node configuration
  cisco.mso.ndo_l3out_floating_svi_interface:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: l3out_template
    l3out: l3out_name
    node_id: 101
    encapsulation_type: vlan
    encapsulation_value: 1200
    state: query
  register: query_one

- name: Query all existing L3Out Floating SVI Interfaces of a L3Out with L3Out Node configuration
  cisco.mso.ndo_l3out_floating_svi_interface:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: l3out_template
    l3out: l3out_name
    state: query
  register: query_all

- name: Delete an existing L3Out Floating SVI Interface
  cisco.mso.ndo_l3out_floating_svi_interface:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: l3out_template
    l3out: l3out_name
    node_id: 101
    encapsulation_type: vlan
    encapsulation_value: 1200
    state: absent
"""

RETURN = r"""
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.mso.plugins.module_utils.mso import MSOModule, mso_argument_spec
from ansible_collections.cisco.mso.plugins.module_utils.templates import MSOTemplates
from ansible_collections.cisco.mso.plugins.module_utils.constants import TARGET_DSCP_MAP
from ansible_collections.cisco.mso.plugins.module_utils.utils import append_update_ops_data, delete_none_values
from ansible_collections.cisco.mso.plugins.module_utils.l3out_node import L3OutNode
import copy


def main():
    argument_spec = mso_argument_spec()
    argument_spec.update(
        template=dict(type="str", aliases=["l3out_template"]),
        template_id=dict(type="str", aliases=["l3out_template_id"]),
        l3out=dict(type="str", aliases=["l3out_name"]),
        l3out_uuid=dict(type="str"),
        node_id=dict(type="str", aliases=["node", "border_leaf"]),
        node_group_policy=dict(type="str"),
        node_router_id=dict(type="str", aliases=["router_id"]),
        use_router_id_as_loopback=dict(type="bool"),
        node_loopback_ip=dict(type="str", aliases=["loopback_ip"]),
        interface_group_policy=dict(type="str"),
        ipv4_address=dict(type="str"),
        ipv6_address=dict(type="str"),
        ipv6_link_local_address=dict(type="str"),
        ipv6_dad=dict(type="str", choices=["enabled", "disabled"]),
        mac=dict(type="str"),
        mtu=dict(type="str"),
        target_dscp=dict(type="str", choices=list(TARGET_DSCP_MAP)),
        encapsulation_scope=dict(type="str", choices=["local", "vrf"], aliases=["encap_scope"]),
        encapsulation_type=dict(type="str", choices=["vlan", "vxlan"], aliases=["encap_type"]),
        encapsulation_value=dict(type="int", aliases=["encap", "encapsulation", "encapsulation_id"]),
        encapsulation_mode=dict(type="str", choices=["access", "trunk", "trunk_native"], aliases=["encap_mode"]),
        auto_state=dict(type="str", choices=["enabled", "disabled"]),
        state=dict(type="str", default="query", choices=["absent", "query", "present"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "present", ["node_id"]],
            ["state", "absent", ["node_id"]],
        ],
        required_by={"node_id": ("encapsulation_type", "encapsulation_value")},
        required_one_of=[
            ["template", "template_id"],
            ["l3out", "l3out_uuid"],
        ],
        mutually_exclusive=[
            ["template", "template_id"],
            ["l3out", "l3out_uuid"],
        ],
    )

    mso = MSOModule(module)
    mso_templates = MSOTemplates(mso)

    template_name = mso.params.get("template")
    template_id = mso.params.get("template_id")
    l3out = mso.params.get("l3out")
    l3out_uuid = mso.params.get("l3out_uuid")
    node_id = mso.params.get("node_id")
    node_router_id = mso.params.get("node_router_id")
    node_group_policy = mso.params.get("node_group_policy")
    use_router_id_as_loopback = mso.params.get("use_router_id_as_loopback")
    node_loopback_ip = mso.params.get("node_loopback_ip")
    interface_group_policy = mso.params.get("interface_group_policy")
    ipv4_address = mso.params.get("ipv4_address")
    ipv6_address = mso.params.get("ipv6_address")
    ipv6_link_local_address = mso.params.get("ipv6_link_local_address")
    ipv6_dad = mso.params.get("ipv6_dad")
    mac = mso.params.get("mac")
    mtu = mso.params.get("mtu")
    target_dscp = mso.params.get("target_dscp")
    encapsulation_scope = mso.params.get("encapsulation_scope")
    encapsulation_type = mso.params.get("encapsulation_type")
    encapsulation_value = mso.params.get("encapsulation_value")
    encap = None
    if encapsulation_type and encapsulation_value:
        encap = {"encapType": encapsulation_type, "value": encapsulation_value}
    encapsulation_mode = mso.params.get("encapsulation_mode")
    if encapsulation_mode == "trunk_native":
        encapsulation_mode = "access8021p"
    auto_state = mso.params.get("auto_state")
    state = mso.params.get("state")

    mso_template = mso_templates.get_template("l3out", template_name, template_id)
    mso_template.validate_template("l3out")
    l3out_object = mso_template.get_l3out_object(l3out_uuid, l3out, True)

    pod_id = None
    if node_id:
        pod_id = mso.get_site_interface_details(
            site_id=mso_template.template.get("l3outTemplate", {}).get("siteId"),
            node=node_id,
        )

    match = mso_template.get_l3out_floating_svi_interface(l3out_object.details, pod_id, node_id, encap)
    if node_id and encap and match:
        set_floating_svi_interface_details(mso_template, match.details, l3out_object)
        mso.existing = mso.previous = copy.deepcopy(match.details)  # Query a specific object
    elif match:
        mso.existing = [set_floating_svi_interface_details(mso_template, obj, l3out_object) for obj in match]

    l3out_interface_path = "/l3outTemplate/l3outs/{0}/floatingSviInterfaces/{1}".format(l3out_object.index, match.index if match else "-")

    ops = []

    if state != "query":
        l3out_node = L3OutNode(mso.params, mso_template, l3out_object, pod_id, node_id)

    if state == "present":
        mso_values = {
            "pathType": "anchorNode",
            "group": interface_group_policy,
            "mac": mac,
            "mtu": mtu,
            "targetDscp": target_dscp,
            "encap": encap,
            "nodeID": node_id,
            "podID": pod_id,
        }

        if match:
            mso_values[("addresses", "primaryV4")] = ipv4_address
            mso_values[("addresses", "primaryV6")] = ipv6_address
            mso_values[("addresses", "linkLocalV6")] = ipv6_link_local_address
            mso_values[("addresses", "ipV6DAD")] = ipv6_dad
            mso_values[("svi", "encapScope")] = encapsulation_scope
            mso_values[("svi", "mode")] = encapsulation_mode
            mso_values[("svi", "autostate")] = auto_state

            mso_values = delete_none_values(mso_values)
            append_update_ops_data(ops, match.details, l3out_interface_path, mso_values)
            mso.sanitize(match.details, collate=True)

        else:
            mso_values["addresses"] = {
                "primaryV4": ipv4_address,
                "primaryV6": ipv6_address,
                "linkLocalV6": ipv6_link_local_address,
                "ipV6DAD": ipv6_dad,
            }

            mso_values["svi"] = {
                "encapScope": encapsulation_scope,
                "mode": encapsulation_mode,
                "autostate": auto_state,
            }

            mso_values = delete_none_values(mso_values)
            mso.sanitize(mso_values)
            ops.append(dict(op="add", path=l3out_interface_path, value=mso.sent))

            # update mso.proposed with interface details that are not included in the interface payload and node details
            mso.proposed["node"] = l3out_node.construct_node_payload()
            set_floating_svi_interface_details(mso_template, mso.proposed, l3out_object)

        l3out_node.update_ops(ops)

    elif state == "absent":
        if match:
            ops.append(dict(op="remove", path=l3out_interface_path))

    if not mso.module.check_mode and ops:
        ignore_errors = ["node {0}-{1} doesn't have an interface configured".format(pod_id, node_id)]
        remove_ops = {ignore_errors[0]: l3out_node.get_node_remove_op()}
        response = mso.l3out_interface_request(mso_template, ops, ignore_errors, state, remove_ops)
        l3out_object = mso_template.get_l3out_object(l3out_uuid, l3out, True, search_object=response)
        match = mso_template.get_l3out_floating_svi_interface(l3out_object.details, pod_id, node_id, encap)
        if match:
            set_floating_svi_interface_details(mso_template, match.details, l3out_object)
            mso.existing = match.details  # When the state is present
        else:
            mso.existing = {}  # When the state is absent
    elif mso.module.check_mode and state != "query":  # When the state is present/absent with check mode
        mso.existing = mso.proposed if state == "present" else {}
    mso.exit_json()


def set_floating_svi_interface_details(mso_template, floating_svi_interface, l3out_object):
    mso_template.update_config_with_port_channel_references(floating_svi_interface)
    mso_template.update_config_with_node_references(floating_svi_interface, l3out_object)
    return floating_svi_interface


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (c) 2013, Darryl Stoflet <stoflet@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: monit
short_description: Manage the state of a program monitored using Monit
description:
  - Manage the state of a program monitored using Monit.
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  name:
    description:
      - The name of the C(monit) program/process to manage.
    required: true
    type: str
  state:
    description:
      - The state of service.
    required: true
    choices: ["present", "started", "stopped", "restarted", "monitored", "unmonitored", "reloaded"]
    type: str
  timeout:
    description:
      - If there are pending actions for the service monitored by monit, then it checks for up to this many seconds to verify
        the requested action has been performed. The module sleeps for five seconds between each check.
    default: 300
    type: int
author:
  - Darryl Stoflet (@dstoflet)
  - Simon Kelly (@snopoke)
requirements:
  - Monit. Support for versions 5.18 and older is deprecated and will be removed in community.general 14.0.0.
"""

EXAMPLES = r"""
- name: Manage the state of program httpd to be in started state
  community.general.monit:
    name: httpd
    state: started
"""

import re
import time
from enum import Enum

from ansible.module_utils.basic import AnsibleModule

STATE_COMMAND_MAP = {
    "stopped": "stop",
    "started": "start",
    "monitored": "monitor",
    "unmonitored": "unmonitor",
    "restarted": "restart",
}

MONIT_SERVICES = ["Process", "File", "Fifo", "Filesystem", "Directory", "Remote host", "System", "Program", "Network"]


class StatusValue(Enum):
    MISSING = "missing"
    OK = "ok"
    NOT_MONITORED = "not_monitored"
    INITIALIZING = "initializing"
    DOES_NOT_EXIST = "does_not_exist"
    EXECUTION_FAILED = "execution_failed"


class Status:
    """Represents a monit status with optional pending state."""

    def __init__(self, status_val: str | StatusValue, is_pending: bool = False):
        if isinstance(status_val, StatusValue):
            self.state = status_val
        else:
            self.state = getattr(StatusValue, status_val)
        self.is_pending = is_pending

    @property
    def value(self):
        return self.state.value

    def pending(self):
        """Return a new Status instance with is_pending=True."""
        return Status(self.state, is_pending=True)

    def __getattr__(self, item):
        if item.startswith("is_"):
            status_name = item[3:].upper()
            if hasattr(StatusValue, status_name):
                return self.value == getattr(StatusValue, status_name).value
        raise AttributeError(item)

    def __eq__(self, other):
        if not isinstance(other, Status):
            return False
        return self.state == other.state and self.is_pending == other.is_pending

    def __str__(self):
        return f"{self.value}{' (pending)' if self.is_pending else ''}"

    def __repr__(self):
        return f"<{self}>"


# Initialize convenience class attributes


class Monit:
    def __init__(self, module, monit_bin_path, service_name, timeout):
        self.module = module
        self.monit_bin_path = monit_bin_path
        self.process_name = service_name
        self.timeout = timeout

        self._monit_version = None
        self._raw_version = None
        self._status_change_retry_count = 6

        if self.monit_version() <= (5, 18):
            module.deprecate(
                "Support for monit 5.18 and older is deprecated and will be removed in community.general 14.0.0.",
                version="14.0.0",
                collection_name="community.general",
            )

    def monit_version(self):
        if self._monit_version is None:
            self._raw_version, version = self._get_monit_version()
            # Use only major and minor even if there are more these should be enough
            self._monit_version = version[0], version[1]
        return self._monit_version

    def _get_monit_version(self):
        rc, out, err = self.module.run_command([self.monit_bin_path, "-V"], check_rc=True)
        version_line = out.split("\n")[0]
        raw_version = re.search(r"([0-9]+\.){1,2}([0-9]+)?", version_line).group()
        return raw_version, tuple(map(int, raw_version.split(".")))

    def exit_fail(self, msg, status=None, **kwargs):
        kwargs.update(
            {
                "msg": msg,
                "monit_version": self._raw_version,
                "process_status": str(status) if status else None,
            }
        )
        self.module.fail_json(**kwargs)

    def exit_success(self, state):
        self.module.exit_json(
            changed=True,
            name=self.process_name,
            monit_version=self._raw_version,
            state=state,
        )

    @property
    def command_args(self):
        return ["-B"] if self.monit_version() > (5, 18) else []

    def get_status(self, validate=False) -> Status:
        """Return the status of the process in monit.

        :@param validate: Force monit to re-check the status of the process
        """
        monit_command = "validate" if validate else "status"
        check_rc = not validate  # 'validate' always has rc = 1
        command = [self.monit_bin_path, monit_command] + self.command_args + [self.process_name]
        rc, out, err = self.module.run_command(command, check_rc=check_rc)
        return self._parse_status(out, err)

    def _parse_status(self, output, err) -> Status:
        escaped_monit_services = "|".join([re.escape(x) for x in MONIT_SERVICES])
        pattern = f"({escaped_monit_services}) '{re.escape(self.process_name)}'"
        if not re.search(pattern, output, re.IGNORECASE):
            return Status("MISSING")

        status_val_find = re.findall(r"^\s*status\s*([\w\- ]+)", output, re.MULTILINE)
        if not status_val_find:
            self.exit_fail("Unable to find process status", stdout=output, stderr=err)

        status_val = status_val_find[0].strip().upper()
        if " | " in status_val:
            status_val = status_val.split(" | ")[0]
        if " - " not in status_val:
            status_val = status_val.replace(" ", "_")
            # Normalize RUNNING to OK (monit reports both, they mean the same thing)
            if status_val == "RUNNING":
                status_val = "OK"
            try:
                return Status(status_val)
            except AttributeError:
                self.module.warn(f"Unknown monit status '{status_val}', treating as execution failed")
                return Status("EXECUTION_FAILED")
        else:
            status_val, substatus = status_val.split(" - ")
            action, state = substatus.split()
            if action in ["START", "INITIALIZING", "RESTART", "MONITOR"]:
                status = Status("OK")
            else:
                status = Status("NOT_MONITORED")

            if state == "PENDING":
                status = status.pending()
            return status

    def is_process_present(self):
        command = [self.monit_bin_path, "summary"] + self.command_args
        rc, out, err = self.module.run_command(command, check_rc=True)
        return bool(re.findall(rf"\b{self.process_name}\b", out))

    def is_process_running(self):
        return self.get_status().is_ok

    def run_command(self, command):
        """Runs a monit command, and returns the new status."""
        return self.module.run_command([self.monit_bin_path, command, self.process_name], check_rc=True)

    def wait_for_status_change(self, current_status):
        running_status = self.get_status()
        if running_status.value != current_status.value or current_status.value == StatusValue.EXECUTION_FAILED:
            return running_status

        loop_count = 0
        while running_status.value == current_status.value:
            if loop_count >= self._status_change_retry_count:
                self.exit_fail("waited too long for monit to change state", running_status)

            loop_count += 1
            time.sleep(0.5)
            validate = loop_count % 2 == 0  # force recheck of status every second try
            running_status = self.get_status(validate)
        return running_status

    def wait_for_monit_to_stop_pending(self, current_status=None):
        """Fails this run if there is no status or it is pending/initializing for timeout"""
        timeout_time = time.time() + self.timeout

        if not current_status:
            current_status = self.get_status()
        waiting_status = [
            StatusValue.MISSING,
            StatusValue.INITIALIZING,
            StatusValue.DOES_NOT_EXIST,
        ]
        while current_status.is_pending or (current_status.state in waiting_status):
            if time.time() >= timeout_time:
                self.exit_fail('waited too long for "pending", or "initiating" status to go away', current_status)

            time.sleep(5)
            current_status = self.get_status(validate=True)
        return current_status

    def reload(self):
        rc, out, err = self.module.run_command([self.monit_bin_path, "reload"])
        if rc != 0:
            self.exit_fail("monit reload failed", stdout=out, stderr=err)
        self.exit_success(state="reloaded")

    def present(self):
        self.run_command("reload")

        timeout_time = time.time() + self.timeout
        while not self.is_process_present():
            if time.time() >= timeout_time:
                self.exit_fail('waited too long for process to become "present"')

            time.sleep(5)

        self.exit_success(state="present")

    def change_state(self, state: str, expected_status: StatusValue, invert_expected: bool | None = None):
        current_status = self.get_status()
        self.run_command(STATE_COMMAND_MAP[state])
        # Give monit daemon a moment to process the command before checking status
        # to avoid race condition where HTTP interface may be temporarily unresponsive
        time.sleep(0.5)
        status = self.wait_for_status_change(current_status)
        status = self.wait_for_monit_to_stop_pending(status)
        status_match = status.state == expected_status
        if invert_expected:
            status_match = not status_match
        if status_match:
            self.exit_success(state=state)
        self.exit_fail(f"{self.process_name} process not {state}", status)

    def stop(self):
        self.change_state("stopped", expected_status=StatusValue.NOT_MONITORED)

    def unmonitor(self):
        self.change_state("unmonitored", expected_status=StatusValue.NOT_MONITORED)

    def restart(self):
        self.change_state("restarted", expected_status=StatusValue.OK)

    def start(self):
        self.change_state("started", expected_status=StatusValue.OK)

    def monitor(self):
        self.change_state("monitored", expected_status=StatusValue.NOT_MONITORED, invert_expected=True)


def main():
    arg_spec = dict(
        name=dict(required=True),
        timeout=dict(default=300, type="int"),
        state=dict(
            required=True,
            choices=["present", "started", "restarted", "stopped", "monitored", "unmonitored", "reloaded"],
        ),
    )

    module = AnsibleModule(argument_spec=arg_spec, supports_check_mode=True)
    module.run_command_environ_update = {"LANGUAGE": "C", "LC_ALL": "C"}

    name = module.params["name"]
    state = module.params["state"]
    timeout = module.params["timeout"]

    monit = Monit(module, module.get_bin_path("monit", True), name, timeout)

    def exit_if_check_mode():
        if module.check_mode:
            module.exit_json(changed=True)

    if state == "reloaded":
        exit_if_check_mode()
        monit.reload()

    present = monit.is_process_present()

    if not present and state != "present":
        module.fail_json(msg=f"{name} process not presently configured with monit", name=name)

    if state == "present":
        if present:
            module.exit_json(changed=False, name=name, state=state)
        exit_if_check_mode()
        monit.present()

    monit.wait_for_monit_to_stop_pending()
    running = monit.is_process_running()

    if running and state in ["started", "monitored"]:
        module.exit_json(changed=False, name=name, state=state)

    if running and state == "stopped":
        exit_if_check_mode()
        monit.stop()

    if running and state == "unmonitored":
        exit_if_check_mode()
        monit.unmonitor()

    elif state == "restarted":
        exit_if_check_mode()
        monit.restart()

    elif not running and state == "started":
        exit_if_check_mode()
        monit.start()

    elif not running and state == "monitored":
        exit_if_check_mode()
        monit.monitor()

    module.exit_json(changed=False, name=name, state=state)


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (c) 2012, Matt Wright <matt@nobien.net>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: easy_install
short_description: Installs Python libraries
description:
  - Installs Python libraries, optionally in a C(virtualenv).
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  name:
    type: str
    description:
      - A Python library name.
    required: true
  virtualenv:
    type: str
    description:
      - An optional O(virtualenv) directory path to install into. If the O(virtualenv) does not exist, it is created automatically.
  virtualenv_site_packages:
    description:
      - Whether the virtual environment inherits packages from the global site-packages directory. Note that this setting
        has no effect on an already existing virtual environment, so if you want to change it, the environment must be deleted
        and newly created.
    type: bool
    default: false
  virtualenv_command:
    type: str
    description:
      - The command to create the virtual environment with. For example V(pyvenv), V(virtualenv), V(virtualenv2).
    default: virtualenv
  executable:
    type: str
    description:
      - The explicit executable or a pathname to the executable to be used to run easy_install for a specific version of Python
        installed in the system. For example V(easy_install-3.3), if there are both Python 2.7 and 3.3 installations in the
        system and you want to run easy_install for the Python 3.3 installation.
    default: easy_install
  state:
    type: str
    description:
      - The desired state of the library. V(latest) ensures that the latest version is installed.
    choices: [present, latest]
    default: present
notes:
  - Please note that the C(easy_install) module can only install Python libraries. Thus this module is not able to remove
    libraries. It is generally recommended to use the M(ansible.builtin.pip) module which you can first install using M(community.general.easy_install).
  - Also note that C(virtualenv) must be installed on the remote host if the O(virtualenv) parameter is specified.
requirements: ["virtualenv"]
author: "Matt Wright (@mattupstate)"
"""

EXAMPLES = r"""
- name: Install or update pip
  community.general.easy_install:
    name: pip
    state: latest

- name: Install Bottle into the specified virtualenv
  community.general.easy_install:
    name: bottle
    virtualenv: /webapps/myapp/venv

- name: Install a python package using pyvenv as the virtualenv tool
  community.general.easy_install:
    name: package_name
    virtualenv: /opt/myenv
    virtualenv_command: pyvenv
"""

import os
import os.path
import tempfile

from ansible.module_utils.basic import AnsibleModule


def install_package(module, name, easy_install, executable_arguments):
    cmd = [easy_install] + executable_arguments + [name]
    rc, out, err = module.run_command(cmd)
    return rc, out, err


def _is_package_installed(module, name, easy_install, executable_arguments):
    # Copy and add to the arguments
    executable_arguments = executable_arguments[:]
    executable_arguments.append("--dry-run")
    rc, out, err = install_package(module, name, easy_install, executable_arguments)
    if rc:
        module.fail_json(msg=err)
    return "Downloading" not in out


def _get_easy_install(module, env=None, executable=None):
    candidate_easy_inst_basenames = ["easy_install"]
    easy_install = None
    if executable is not None:
        if os.path.isabs(executable):
            easy_install = executable
        else:
            candidate_easy_inst_basenames.insert(0, executable)
    if easy_install is None:
        if env is None:
            opt_dirs = []
        else:
            # Try easy_install with the virtualenv directory first.
            opt_dirs = [f"{env}/bin"]
        for basename in candidate_easy_inst_basenames:
            easy_install = module.get_bin_path(basename, False, opt_dirs)
            if easy_install is not None:
                break
    # easy_install should have been found by now.  The final call to
    # get_bin_path will trigger fail_json.
    if easy_install is None:
        basename = candidate_easy_inst_basenames[0]
        easy_install = module.get_bin_path(basename, True, opt_dirs)
    return easy_install


def main():
    arg_spec = dict(
        name=dict(required=True),
        state=dict(default="present", choices=["present", "latest"], type="str"),
        virtualenv=dict(),
        virtualenv_site_packages=dict(default=False, type="bool"),
        virtualenv_command=dict(default="virtualenv"),
        executable=dict(default="easy_install"),
    )

    module = AnsibleModule(argument_spec=arg_spec, supports_check_mode=True)
    module.run_command_environ_update = {"LANGUAGE": "C", "LC_ALL": "C"}

    name = module.params["name"]
    env = module.params["virtualenv"]
    executable = module.params["executable"]
    site_packages = module.params["virtualenv_site_packages"]
    virtualenv_command = module.params["virtualenv_command"]
    executable_arguments = []
    if module.params["state"] == "latest":
        executable_arguments.append("--upgrade")

    rc = 0
    err = ""
    out = ""

    if env:
        virtualenv = module.get_bin_path(virtualenv_command, True)

        if not os.path.exists(os.path.join(env, "bin", "activate")):
            if module.check_mode:
                module.exit_json(changed=True)
            command = f"{virtualenv} {env}"
            if site_packages:
                command += " --system-site-packages"
            cwd = tempfile.gettempdir()
            rc_venv, out_venv, err_venv = module.run_command(command, cwd=cwd)

            rc += rc_venv
            out += out_venv
            err += err_venv

    easy_install = _get_easy_install(module, env, executable)

    cmd = None
    changed = False
    installed = _is_package_installed(module, name, easy_install, executable_arguments)

    if not installed:
        if module.check_mode:
            module.exit_json(changed=True)
        rc_easy_inst, out_easy_inst, err_easy_inst = install_package(module, name, easy_install, executable_arguments)

        rc += rc_easy_inst
        out += out_easy_inst
        err += err_easy_inst

        changed = True

    if rc != 0:
        module.fail_json(msg=err, cmd=cmd)

    module.exit_json(changed=changed, binary=easy_install, name=name, virtualenv=env)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-

#  Copyright 2017 Michael De La Rue | Ansible
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""This module adds shared support for generic Amazon AWS modules

In order to use this module, include it as part of a custom
module as shown below.

  from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
  module = AnsibleAWSModule(argument_spec=dictionary, supports_check_mode=boolean
                            mutually_exclusive=list1, required_together=list2)

The 'AnsibleAWSModule' module provides similar, but more restricted,
interfaces to the normal Ansible module.  It also includes the
additional methods for connecting to AWS using the standard module arguments

    m.resource('lambda') # - get an AWS connection as a boto3 resource.

or

    m.client('sts') # - get an AWS connection as a boto3 client.

To make use of AWSRetry easier, it can now be wrapped around any call from a
module-created client. To add retries to a client, create a client:

    m.client('ec2', retry_decorator=AWSRetry.jittered_backoff(retries=10))

Any calls from that client can be made to use the decorator passed at call-time
using the `aws_retry` argument. By default, no retries are used.

    ec2 = m.client('ec2', retry_decorator=AWSRetry.jittered_backoff(retries=10))
    ec2.describe_instances(InstanceIds=['i-123456789'], aws_retry=True)

The call will be retried the specified number of times, so the calling functions
don't need to be wrapped in the backoff decorator.
"""

# Used to live here, moved into ansible_collections.amazon.aws.plugins.module_utils.arn
from .arn import parse_aws_arn  # pylint: disable=unused-import

# Used to live here, moved into ansible_collections.amazon.aws.plugins.module_utils.botocore
from .botocore import HAS_BOTO3  # pylint: disable=unused-import
from .botocore import get_boto3_client_method_parameters  # pylint: disable=unused-import
from .botocore import is_boto3_error_code  # pylint: disable=unused-import
from .botocore import is_boto3_error_message  # pylint: disable=unused-import
from .botocore import normalize_boto3_result  # pylint: disable=unused-import

# Used to live here, moved into ansible_collections.amazon.aws.plugins.module_utils.exceptions
from .exceptions import AnsibleAWSError  # pylint: disable=unused-import

# Used to live here, moved into ansible_collections.amazon.aws.plugins.module_utils.modules
from .modules import AnsibleAWSModule  # pylint: disable=unused-import

# Used to live here, moved into ansible_collections.amazon.aws.plugins.module_utils.modules
from .transformation import scrub_none_parameters  # pylint: disable=unused-import

# We will also export HAS_BOTO3 so end user modules can use it.
__all__ = ("AnsibleAWSModule", "HAS_BOTO3", "is_boto3_error_code", "is_boto3_error_message")



# -*- coding: utf-8 -*-
# Copyright 2019 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
"""
The eos l2_interfaces fact class
It is in this file the configuration is collected from the device
for a given resource, parsed, and the facts tree is populated
based on the configuration.
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type

import re

from copy import deepcopy

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common import utils

from ansible_collections.arista.eos.plugins.module_utils.network.eos.argspec.l2_interfaces.l2_interfaces import (
    L2_interfacesArgs,
)


class L2_interfacesFacts(object):
    """The eos l2_interfaces fact class"""

    def __init__(self, module, subspec="config", options="options"):
        self._module = module
        self.argument_spec = L2_interfacesArgs.argument_spec
        spec = deepcopy(self.argument_spec)
        if subspec:
            if options:
                facts_argument_spec = spec[subspec][options]
            else:
                facts_argument_spec = spec[subspec]
        else:
            facts_argument_spec = spec

        self.generated_spec = utils.generate_dict(facts_argument_spec)

    def get_device_data(self, connection):
        return connection.get("show running-config | section ^interface")

    def populate_facts(self, connection, ansible_facts, data=None):
        """Populate the facts for l2_interfaces

        :param connection: the device connection
        :param ansible_facts: Facts dictionary
        :param data: previously collected configuration
        :rtype: dictionary
        :returns: facts
        """
        if not data:
            data = self.get_device_data(connection)

        # operate on a collection of resource x
        config = ("\n" + data).split("\ninterface ")
        objs = []
        for conf in config:
            if conf:
                obj = self.render_config(self.generated_spec, conf)
                if obj:
                    objs.append(obj)
        facts = {}
        if objs:
            params = utils.validate_config(
                self.argument_spec,
                {"config": objs},
            )
            facts["l2_interfaces"] = [utils.remove_empties(cfg) for cfg in params["config"]]
        ansible_facts["ansible_network_resources"].update(facts)

        return ansible_facts

    def render_config(self, spec, conf):
        """
        Render config as dictionary structure and delete keys from spec for null values

        :param spec: The facts tree, generated from the argspec
        :param conf: The configuration
        :rtype: dictionary
        :returns: The generated config
        """
        config = deepcopy(spec)

        # populate the facts from the configuration
        config["name"] = re.match(r"(\S+)", conf).group(1).replace('"', "")
        has_mode = re.search(r"switchport mode (\S+)", conf)
        if has_mode:
            config["mode"] = has_mode.group(1)

        has_access = re.search(r"switchport access vlan (\d+)", conf)
        if has_access:
            config["access"] = {"vlan": int(has_access.group(1))}

        has_trunk = re.findall(r"switchport trunk (.+)", conf)
        if has_trunk:
            trunk = {}
            for match in has_trunk:
                has_native = re.match(r"native vlan (\d+)", match)
                if has_native:
                    trunk["native_vlan"] = int(has_native.group(1))
                    continue

                has_allowed = re.match(r"allowed vlan (\S+)", match)
                if has_allowed:
                    # TODO: listify?
                    trunk["trunk_allowed_vlans"] = has_allowed.group(1)
                    continue
            config["trunk"] = trunk

        return utils.remove_empties(config)
